﻿using Buddy.Common;
using Buddy.Wildstar.Game;
using log4net;
using MahApps.Metro.Controls;
using System.IO;
using System.Windows;
using System.Windows.Markup;
using System;

namespace Buddy.Wildstar.Plugins.WBTimerKill
{
    class WBTimerSettings : JsonSettings
    {
        private Flyout mainFlyout;
        private MetroWindow MainWindow
        {
            get
            {
                return (MetroWindow)App.Current.MainWindow;
            }
        }
        private ILog Log = LogManager.GetLogger("WBTimerSettings");

        // Constructor.
        public WBTimerSettings()
            : base(GetSettingsFilePath(SettingsPath, "WBTimer", GameManager.LocalPlayer.Name, "Settings.json"))
        {
            string[] files = Directory.GetFiles(AppSettings.Instance.FullPluginsPath, "WBTimerSettings.xaml", SearchOption.AllDirectories);
            using (var fs = new FileStream(files[0], FileMode.Open))
            {
                this.mainFlyout = (Flyout)XamlReader.Load(fs);
                (this.mainFlyout as FrameworkElement).DataContext = this;
            }
        }

        // Singleton.
        private static WBTimerSettings _instance;
        public static WBTimerSettings Instance
        {
            get
            {
                if (_instance == null)
                    _instance = new WBTimerSettings();

                return _instance;
            }
        }

        public static void Refresh()
        {
            if (_instance != null)
                _instance.Remove();

            _instance = new WBTimerSettings();
        }

        // Methods.
        public void Toggle()
        {
            if (this.mainFlyout.IsOpen)
                this.Hide();
            else
                this.Show();
        }
        private void Add()
        {
            if (!MainWindow.Flyouts.Items.Contains(this.mainFlyout))
                MainWindow.Flyouts.Items.Add(this.mainFlyout);
        }
        private void Remove()
        {
            if (MainWindow.Flyouts.Items.Contains(this.mainFlyout))
                MainWindow.Flyouts.Items.Remove(this.mainFlyout);
        }
        private void Show()
        {
            if (!MainWindow.Flyouts.Items.Contains(this.mainFlyout))
                this.Add();

            this.mainFlyout.IsOpen = true;
        }
        private void Hide()
        {
            this.mainFlyout.IsOpen = false;
        }

        // General.

        #region Timer Values

        private int _timerValue = 120;
        public int timerValue
        {
            get { return _timerValue; }

            set
            {
                if (value.Equals(_timerValue))
                    return;

                _timerValue = value;
                Log.Debug("Timer Set to: " + _timerValue.ToString());
                NotifyPropertyChanged(() => timerValue);
                base.Save();
            }
        }

        #endregion

        #region Enabled/Disable

        private bool _enableDisable;
        public bool enableDisable
        {
            get { return _enableDisable; }

            set
            {
                if (value.Equals(_enableDisable))
                    return;

                _enableDisable = value;
                Log.Debug("Timer status set to: " + _enableDisable.ToString());
                NotifyPropertyChanged(() => enableDisable);
                base.Save();
            }
        }

        #endregion
    }
}
