﻿using Buddy.Wildstar.Engine;
using Buddy.Wildstar.Game;
using Buddy.Wildstar.Game.Actors;
using Buddy.Wildstar.BotCommon;
using log4net;
using System;
using System.Diagnostics;
using System.Threading.Tasks;
using System.Net.Mail;
using Buddy.Common.Math;
using Buddy.Coroutines;

namespace Buddy.Wildstar.Plugins.WBTimerKill
{
    public class WBTimerKill : CoroutinePulsable, IPlugin, IUIButtonProvider
    {
        public string Name { get { return "Stop Timer"; } }
        public string Author { get { return "themarketguy"; } }
        public Version Version { get { return new Version("0.0.1"); } }
        public override bool CanBePulsed { get { return true; } }

        #region IPlugin Implementation

        private static ILog Log = LogManager.GetLogger("WBTimer");

        private bool isEnabled;
        public bool Enabled
        {
            get { return isEnabled; }
            set { isEnabled = value; }
        }

        public void Initialize()
        {
            GameEngine.BotPulsator.RegisterForPulse(this);
            WBTimerSettings.Refresh();
            WBTimer = new Stopwatch();
            WBTimerSettings.Instance.enableDisable = false;
        }

        public static Stopwatch WBTimer;

        public void Uninitialize()
        {
            GameEngine.BotPulsator.UnregisterForPulse(this);
            WBTimer.Stop();
            WBTimer.Reset();
            WBTimerSettings.Instance.enableDisable = false;
        }

        #endregion

        #region IUIButtonProvder Implementation

        public string ButtonText
        {
            get { return "Settings"; }
        }

        public void OnButtonClicked(object sender)
        {
            WBTimerSettings.Instance.Toggle();
        }

        #endregion

        #region Extra Declarations
        private static Player Me
        {
            get
            {
                return GameManager.LocalPlayer;
            }
        }
        private static SpellManager.SpellInformation NextReturnSpell
        {
            get
            {
                SpellManager.SpellInformation spell;
                // Teleport City.
                spell = Me.IsExile ? SpellManager.GetMiscSpell(MiscSpellType.TeleportExileCity) : SpellManager.GetMiscSpell(MiscSpellType.TeleportDominionCity);
                if (spell != null && spell.IsValid && spell.CanCast)
                    return spell;

                // Teleport City.
                spell = SpellManager.GetMiscSpell(MiscSpellType.RecallTransmat);
                if (spell != null && spell.IsValid && spell.CanCast)
                    return spell;

                return null;
            }
        }

        private static async Task Return()
        {
            Navigator.Stop(true);
            GameEngine.BotPulsator.Suspend(GameEngine.CurrentBot, TimeSpan.FromMilliseconds(15000));

            SpellManager.SpellInformation spell = NextReturnSpell;

            if (spell == null)
                return;

            Log.InfoFormat("Casting spell : {0}", spell.Name);
            spell.Cast();

            await CommonBehaviors.WaitForCastCompletion(1000, 15000);
        }

        #endregion

        #region The Goods

        public override async Task CoroutineImplementation()
        {
            if(!WBTimer.IsRunning && WBTimerSettings.Instance.enableDisable)
            {
                WBTimer.Start();
                Log.Debug("WBTimer: Timer Stop enabled, stopwatch started.");
            }

            if(WBTimer.ElapsedMilliseconds >= TimeSpan.FromMinutes(WBTimerSettings.Instance.timerValue).TotalMilliseconds)
            {
                while(Me.IsInCombat)
                {
                    await Coroutine.Sleep(1);
                }
                GameEngine.BotPulsator.Stop();
                Log.Info("Time has expired, we're terminating this botting session.");
                await Return();
                WBTimer.Reset();
                WBTimerSettings.Instance.enableDisable = false;
            }
        }
        
        #endregion
    }
}
